// src/utils.ts
function debounce(func, {
  delay = 1500
} = {}) {
  let timer = void 0;
  return function(...args) {
    !timer && func.apply(this, args);
    timer = setTimeout(() => {
      timer = void 0;
    }, delay);
  };
}

// src/index.ts
function createPlainShiki(shiki) {
  const isSupported = () => "CSS" in globalThis && "highlights" in CSS;
  function mount(el, options) {
    const {
      lang,
      themes = {
        light: "min-light",
        dark: "min-dark"
      },
      selector = (theme) => `.${theme}`,
      watch = true,
      delay = 0
    } = options;
    const debouncedUpdate = debounce(update, { delay });
    const dispose = () => el.removeEventListener("input", debouncedUpdate);
    if (isSupported() && watch) {
      el.addEventListener("input", debouncedUpdate);
    }
    const stylesheet = new CSSStyleSheet();
    document.adoptedStyleSheets.push(stylesheet);
    const colorRanges = /* @__PURE__ */ new Map();
    isSupported() && update();
    function patch(loads) {
      const deleted = /* @__PURE__ */ new Set();
      for (const { token, range } of loads) {
        for (const theme in token.variants) {
          const { color } = token.variants[theme];
          const name = `shiki-${theme}-${color.slice(1).toLowerCase()}`;
          const isDefault = theme === "light";
          let highlight = CSS.highlights.get(name);
          if (!highlight) {
            CSS.highlights.set(name, highlight = new Highlight());
            highlight.priority = isDefault ? 0 : 1;
          }
          let ranges = colorRanges.get(name);
          if (!ranges) {
            const rule = `${isDefault ? ":root" : selector(theme)}::highlight(${name}) { color: ${color}; }`;
            stylesheet.insertRule(rule);
            colorRanges.set(name, ranges = []);
          }
          if (!deleted.has(name)) {
            deleted.add(name);
            for (const range2 of ranges) {
              highlight.delete(range2);
            }
            ranges.length = 0;
          }
          highlight.add(range);
          ranges.push(range);
        }
      }
    }
    function update() {
      const childNodes = el.childNodes;
      const textContent = [...childNodes].map((node) => node.textContent).join("");
      const tokenLines = shiki.codeToTokensWithThemes(textContent, {
        lang,
        themes
      });
      const loads = [];
      for (const tokens of tokenLines) {
        for (const token of tokens) {
          const [node, offset] = findNodeAndOffset(token.offset);
          const range = document.createRange();
          range.setStart(node, offset);
          range.setEnd(node, offset + token.content.length);
          loads.push({ token, range });
        }
      }
      patch(loads);
      function findNodeAndOffset(tokenOffset) {
        let offset = 0;
        for (const node of childNodes) {
          if (!node.textContent) {
            continue;
          }
          if (offset + node.textContent.length > tokenOffset) {
            return [node, tokenOffset - offset];
          } else {
            offset += node.textContent.length;
          }
        }
      }
    }
    return {
      dispose,
      update
    };
  }
  return {
    get isSupported() {
      return isSupported();
    },
    mount
  };
}
export {
  createPlainShiki
};
